﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using emanual.Wpf.Controls; // TreeViewHelper
using System.Windows.Media; // VisualTreeHelper
using System.Diagnostics;

namespace TreeMoveItemTest
{
	public partial class MainWindow : Window
	{
		private Point FMouseDownPosition;
		private TreeViewItem FDragItem = null, FTargetItem = null;
		TreeItemScroll FTreeScroll;

		//---------------------------------------------------------------------------------------------
		public MainWindow()
		{
			InitializeComponent();
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			System.Text.Encoding encoding = emanual.Wpf.Utility.TextEncoding.GetEncodingFromFile(@"ItemData.txt");
			TreeViewHelper.LoadFromFile(treeView, @"ItemData.txt", encoding);

			TreeViewHelper.ExpandAll(treeView);
		}

		//---------------------------------------------------------------------------------------------
		private void treeView_PreviewMouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			// treeVoew 内でマウスダウンした位置
			if ((Keyboard.Modifiers == ModifierKeys.Shift) &&  (e.LeftButton == MouseButtonState.Pressed))
			{
				FMouseDownPosition = e.GetPosition(treeView);
			}
		}

		//---------------------------------------------------------------------------------------------
		private void treeViewItem_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if ((Keyboard.Modifiers == ModifierKeys.Shift) && (e.LeftButton == MouseButtonState.Pressed))
			{
				Point p = e.GetPosition(treeView);

				// マウスダウン位置より X または Y 方向に 3 ピクセル以上移動したときをドラッグ操作を開始したと判断
				if ((Math.Abs(p.X - FMouseDownPosition.X) > 3.0) || (Math.Abs(p.Y - FMouseDownPosition.Y) > 3.0))
				{
					FDragItem = treeView.SelectedItem as TreeViewItem;

					if (FDragItem != null)
					{
						FTreeScroll = new TreeItemScroll(treeView);
						FTreeScroll.Start();

						DragDropEffects dragDropEffect = DragDrop.DoDragDrop(treeView, FDragItem, DragDropEffects.Move);

						// ドロップ後の処理
						if ((dragDropEffect == DragDropEffects.Move) && (FTargetItem != null))
						{
							this.MoveTargetItem(FDragItem, FTargetItem);
							FTargetItem = null;
							FDragItem = null;
						}
					}
				}

				e.Handled = true;
			}
		}

		//---------------------------------------------------------------------------------------------
		private void treeViewItem_DragOver(object sender, DragEventArgs e)
		{
			if (FDragItem != null)
			{
				Point p = e.GetPosition(treeView);

				if (FTreeScroll != null)
					FTreeScroll.Execute(p);
			}

			var item = TreeViewHelper.GetItemContainer(treeView, e.OriginalSource as FrameworkElement);

			if (item != null)
			{
				e.Effects = DragDropEffects.Move;
			}
			else
				e.Effects = DragDropEffects.None;
		}

		//---------------------------------------------------------------------------------------------
		private void treeViewItem_Drop(object sender, DragEventArgs e)
		{
			// ドロップ先が有効かどうかをチェックする
			var targetItem = TreeViewHelper.GetItemContainer(treeView, e.OriginalSource as FrameworkElement);

			if (targetItem != null && FDragItem != null)
			{
				FTargetItem = targetItem as TreeViewItem;
				e.Effects = DragDropEffects.Move;

				if (targetItem == FDragItem)
					e.Effects = DragDropEffects.None;

				if (FTreeScroll != null)
				{
					FTreeScroll.Stop();
					FTreeScroll = null;
				}
			}
		}

		//---------------------------------------------------------------------------------------------
		private void treeViewItem_DragEnter(object sender, DragEventArgs e)
		{
			if (FTargetItem != null && FDragItem != null)
			{
				e.Effects = DragDropEffects.Move;
			}
		}

		//---------------------------------------------------------------------------------------------
		// 実際にデータオブジェクトを移動する（常に移動先の項目の前（同じレベルの上側）に挿入する）
		private void MoveTargetItem(TreeViewItem data, TreeViewItem targetItem)
		{
			if (data != null)
			{
				TreeViewHelper.Remove(treeView, data);
				TreeViewHelper.InsertBefore(treeView, targetItem, data);
				data.IsSelected = true;
			}
		}

		//---------------------------------------------------------------------------------------------
		private T GetFirstVisualChild<T>(Visual obj) where T : Visual
		{
			if (obj != null)
			{
				for (int i = 0; i < VisualTreeHelper.GetChildrenCount(obj); ++i)
				{
					Visual child = (Visual)VisualTreeHelper.GetChild(obj, i);

					if (child != null && child is T)
					{
						return (T)child;
					}

					T childItem = GetFirstVisualChild<T>(child);

					if (childItem != null)
					{
						return childItem;
					}
				}
			}

			return null;
		}
	}
}
