﻿// TreeView コントロールの項目をマウスを使って移動する機能を支援するクラス
using System;
using System.Windows;
//using System.Collections.Generic;
using System.Windows.Controls;
using System.Windows.Media; // VisualTreeHelper
using System.Diagnostics;

namespace emanual.Wpf.Controls
{
	internal class TreeItemScroll
	{
		private const int HEIGHT = 50;      // 領域の高さ
		private Rect FTopRect;              // TreeView 上端部にマウスが入るとスクロールを開始する領域
		private Rect FBottomRect;           // 同、下端部の流域
		private ScrollLine FScrollLine;     // スクロールの要否あるいは方向
		private ScrollViewer FScrollViewer; // TreeView が内蔵する ScrollViewer コントロール
		private System.Windows.Threading.DispatcherTimer FTimer; // スクロールするためのタイマー

		//---------------------------------------------------------------------------------------------
		public TreeItemScroll(TreeView treeView)
		{
			FTopRect = new Rect(0, 0, treeView.ActualWidth, HEIGHT);
			FBottomRect = new Rect(0, treeView.ActualHeight - HEIGHT, treeView.ActualWidth, HEIGHT);

			FTimer = new System.Windows.Threading.DispatcherTimer();
			FTimer.Interval = new TimeSpan(0, 0, 0, 0, 200); // スクロールは 200 ミリ秒ごと
			FTimer.Tick += new EventHandler(timer_Tick);

			FScrollViewer = this.GetFirstVisualChild<ScrollViewer>(treeView);
		}

		//---------------------------------------------------------------------------------------------
		// マウスの現在位置に基づいて必要ならスクロールする（TreeViewItem の DragOver イベントから呼び出す）
		// point : マウスの現在位置
		public void Execute(Point point)
		{
			if (FTopRect.Contains(point)) // TreeView の上端部にあるとき
				FScrollLine = ScrollLine.LineUp;
			else if (FBottomRect.Contains(point)) // TreeView の下端部にあるとき
				FScrollLine = ScrollLine.LineDown;
			else
				FScrollLine = ScrollLine.None;
		}

		//---------------------------------------------------------------------------------------------
		// タイマーを起動する
		public void Start()
		{
			FTimer.Start();
		}

		//---------------------------------------------------------------------------------------------
		// タイマーを停止する
		public void Stop()
		{
			FTimer.Stop();
		}

		//---------------------------------------------------------------------------------------------
		// タイマーによってスクロールを実行する
		private void timer_Tick(object sender, EventArgs e)
		{
			switch (FScrollLine)
			{
				case ScrollLine.LineDown:
					FScrollViewer.LineDown();
					break;
				case ScrollLine.LineUp:
					FScrollViewer.LineUp();
					break;
				default:
					break;
			}
		}

		//---------------------------------------------------------------------------------------------
		// TreeView が内蔵する ScrollViewer コントロールを取得する
		private T GetFirstVisualChild<T>(Visual obj) where T : Visual
		{
			if (obj != null)
			{
				for (int i = 0; i < VisualTreeHelper.GetChildrenCount(obj); ++i)
				{
					Visual child = (Visual)VisualTreeHelper.GetChild(obj, i);

					if (child != null && child is T)
					{
						return (T)child;
					}

					T childItem = GetFirstVisualChild<T>(child);

					if (childItem != null)
					{
						return childItem;
					}
				}
			}

			return null;
		}
	
		//*********************************************************************************************
		// スクロールする方向
		internal enum ScrollLine
		{
			None,
			LineUp,
			LineDown
		}

	} // end of TreeItemScroll class
} // end of TreeItemScroll namespace
